﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace SiloCSharp
{
    class Silo
    {
        private int siloNr;

        public int SiloNr
        {
            get { return siloNr; }
            set { siloNr = value; }
        }
        private double kapazitaet;

        /// <summary>
        /// Wert darf nur gelesen, und nach Initialisierung nicht mehr geändert werden.
        /// </summary>
        /// <value>Menge, die ein Silo fasst</value>
        public double Kapazitaet
        {
            get { return kapazitaet; }
            
        }
        private double bestand;

        /// <summary>
        /// Aktuelle Füllmenge des Silos in Tonnen.
        /// </summary>
        public double Bestand
        {
            get { return bestand; }
            set { bestand = value; }
        }

        /// <summary>
        /// Konstruktor in dem alle Felder gesetzt werden können.
        /// </summary>
        /// <remarks>leerer Konstruktor laut Pflichtenheft nicht zulässig</remarks>
        /// <returns>keine</returns>
        public Silo(int pSiloNr, double pKapazitaet, double pBestand):this(pSiloNr,pKapazitaet)
        {
            //Verkettung von Konstruktoren
            bestand = Math.Max(pBestand, 0);
        }

        /// <summary>
        /// /F15/ Konstruktor sieht mindestens die Festlegung der SiloNr und der Kapazität vor.
        /// </summary>
        /// <remarks>einen leeren Konstruktor darf es laut Pflichtenheft nicht geben.</remarks>
        /// <param name="pSiloNr">ganzzahliger Wert</param>
        /// <param name="pKapazitaet">positiver Wert</param>
        /// <returns>keine</returns>
        public Silo(int pSiloNr, double pKapazitaet)
        {
            siloNr = Math.Max(pSiloNr,0);
            kapazitaet = Math.Max(pKapazitaet,0);
        }

        /// <summary>
        /// /F40/ In ein Silo können mit Hilfe dieser Methode zusätzliche Mengen eingelagert werden. Würde durch das Einlagern die Kapazität überschritten, wird der Vorgang nicht durchgeführt und entsprechend protokolliert.
        /// </summary>
        /// <remarks>Teileinlagerung ist nicht vorgesehen. Ein Auftrag ist ganz oder gar nicht abzuwickeln.</remarks>
        /// <param name="pMenge">positiver Wert</param>
        /// <returns>Text mit "Aktion erfolgreich durchgeführt" bzw. "Aktion nicht ausgeführt, da Kapazität nicht ausreichend"</returns>
        public string einlagern(double pMenge)
        {
            pMenge = Math.Max(pMenge, 0);
            string Ausgabe = "Aktion erfolgreich";
            if (pMenge <= this.restBerechnen())
            {
                bestand += pMenge;
            }
            else
            {
                Ausgabe = "Aktion nicht ausgeführt, da Kapazität nicht ausreichend";
                
            }
            return Ausgabe;
            
           
        }

        /// <summary>
        /// /F50/ Bei dieser Methode reduziert sich der Bestand um die übergebene Menge, es sei denn es soll mehr ausgelagert werden, als vorrätig ist.
        /// </summary>
        /// <remarks>Aktion wird vollständig oder gar nicht ausgeführt!</remarks>
        /// <param name="pMenge">positiver Wert</param>
        /// <returns>Text "Aktion erfolgreich" bzw. "Aktion wurde nicht durchgeführt, es sollte mehr als der vorhandene Bestand ausgelagert werden".</returns>
        public string auslagern(double pMenge)
        {
            pMenge = Math.Max(pMenge, 0);
            string Ausgabe = "Aktion erfolgreich";
            if (pMenge <= bestand)
            {
                bestand -= pMenge;
            }
            else
            {
                Ausgabe = "Aktion wurde nicht durchgeführt, es sollte mehr als der vorhandene Bestand ausgelagert werden";
            }
            return Ausgabe;
        }

        /// <summary>
        /// Aus dem angesprochenen Silo wird die übergeben Menge in das ebenfalls angegeben Zielsilo übertragen, sofern dessen Kapazität ausreicht und das abgebende Silo hinreichend Bestand aufweist.
        /// </summary>
        /// <returns>Text "Aktion erfolgreich" bzw. "Aktion nicht erfolgt , da Bestand im abgebenden Silo nicht ausreichend" oder "Aktion nicht erfolgt, da Aufnahmekapazität des Zielsilos unzureichend.</returns>
        public string umlagern(Silo Ziel, double pMenge)
        {
            string Ausgabe = "Ausgabe erfolgreich";
            if ((pMenge <= Ziel.restBerechnen()) && pMenge <= bestand)
            {
                this.auslagern(pMenge);
                Ziel.einlagern(pMenge);
            }
            else
            {
                Ausgabe = "Aktion nicht durchgeführt, Restriktionen verletzt";
            }
            return Ausgabe;
        }

        /// <summary>
        /// /F70/
        /// Reduktion des Lagerbestandes durch Trocknen um einen übergebenen Prozentsatz.
        /// </summary>
        /// <param name="pSchwund">Prozentsatz zwischen 0 und 100</param>
        public void trocknen(double pSchwund)
        {
            pSchwund = Math.Min(Math.Max(0, pSchwund), 100);
            /* Hier wird gesichert, dass nur Prozentsätze zwischen 
             * 0 und 100 verwendet werden.  */
            this.auslagern((pSchwund / 100) * bestand);
        }

        /// <summary>
        /// Zur Prüfung der Restriktionen, wird hier die noch mögliche Nachfüllmenge des Silos berechnet.
        /// </summary>
        /// <returns>mögliche Restmenge= Kapazität- Bestand</returns>
        public double restBerechnen()
        {
            return kapazitaet - bestand;
        }

        public override string ToString()
        {
            string Ausgabe="";
            Ausgabe = "SiloNr: " + siloNr.ToString() + " \n\r";
            Ausgabe = Ausgabe + "Kapazität: " + kapazitaet.ToString() + " \n\r";
            Ausgabe = Ausgabe + "Bestand: " + bestand.ToString() ;
            return Ausgabe;


        }
    }
}
